@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import argparse
@@ -7,9 +7,11 @@ import sys
 import threading
 
 from UM.Controller import Controller
+from UM.FileProvider import FileProvider
 from UM.Message import Message #For typing.
 from UM.PackageManager import PackageManager
 from UM.PluginRegistry import PluginRegistry
+from UM.Qt.Bindings.FileProviderModel import FileProviderModel
 from UM.Resources import Resources
 from UM.Operations.OperationStack import OperationStack
 from UM.Event import CallFunctionEvent
@@ -79,12 +81,14 @@ class Application:
         self.default_theme = self._app_name  # type: str # Default theme is the application name
         self._default_language = "en_US"  # type: str
 
-        self.change_log_url = "https://github.com/Ultimaker/Uranium"  # Where to find a more detailed description of the recent updates.
+        self.change_log_url: str = "https://github.com/Ultimaker/Uranium"  # Where to find a more detailed description of the recent updates.
+        self.beta_change_log_url: str = "https://github.com/Ultimaker/Uranium"  # Where to find a more detailed description of proposed updates.
 
         self._preferences_filename = None  # type: str
         self._preferences = None  # type: Preferences
 
         self._extensions = []  # type: List[Extension]
+        self._file_providers = []  # type: List[FileProvider]
         self._required_plugins = []  # type: List[str]
 
         self._package_manager_class = PackageManager  # type: type
@@ -95,6 +99,8 @@ class Application:
         self._container_registry = None  # type: ContainerRegistry
         self._global_container_stack = None  # type: Optional[ContainerStack]
 
+        self._file_provider_model = FileProviderModel(application = self)  # type: Optional[FileProviderModel]
+
         self._controller = None  # type: Controller
         self._backend = None  # type: Backend
         self._output_device_manager = None  # type: OutputDeviceManager
@@ -105,14 +111,25 @@ class Application:
 
         self._app_install_dir = self.getInstallPrefix()  # type: str
 
+        # Intended for keeping plugin workspace metadata that is going to be saved in and retrieved from workspace files.
+        # When the workspace is stored, all workspace readers will need to ensure that the workspace metadata is correctly
+        # stored to the output file. The same also holds when loading a workspace; the existing data will be cleared
+        # and replaced with the data recovered from the file (if any).
         self._workspace_metadata_storage = WorkspaceMetadataStorage()  # type: WorkspaceMetadataStorage
 
+        # Intended for keeping plugin workspace information that is only temporary. The information added in this structure
+        # is NOT saved to and retrieved from workspace files.
+        self._current_workspace_information = WorkspaceMetadataStorage()  # type: WorkspaceMetadataStorage
+
     def getAPIVersion(self) -> "Version":
         return self._api_version
 
     def getWorkspaceMetadataStorage(self) -> WorkspaceMetadataStorage:
         return self._workspace_metadata_storage
 
+    def getCurrentWorkspaceInformation(self) -> WorkspaceMetadataStorage:
+        return self._current_workspace_information
+
     # Adds the command line options that can be parsed by the command line parser.
     # Can be overridden to add additional command line options to the parser.
     def addCommandLineOptions(self) -> None:
@@ -173,6 +190,7 @@ class Application:
         PluginRegistry.addType("backend", self.setBackend)
         PluginRegistry.addType("logger", Logger.addLogger)
         PluginRegistry.addType("extension", self.addExtension)
+        PluginRegistry.addType("file_provider", self.addFileProvider)
 
         self._preferences = Preferences()
         self._preferences.addPreference("general/language", self._default_language)
@@ -450,6 +468,12 @@ class Application:
     def getExtensions(self) -> List["Extension"]:
         return self._extensions
 
+    def addFileProvider(self, file_provider: "FileProvider") -> None:
+        self._file_providers.append(file_provider)
+
+    def getFileProviders(self) -> List["FileProvider"]:
+        return self._file_providers
+
     # Returns the path to the folder of the app itself, e.g.: '/root/blah/programs/Cura'.
     @staticmethod
     def getAppFolderPrefix() -> str:
@@ -457,7 +481,10 @@ class Application:
             executable = sys.argv[0]
         else:
             executable = sys.executable
-        return os.path.dirname(os.path.realpath(executable))
+        try:
+            return os.path.dirname(os.path.realpath(executable))
+        except EnvironmentError:  # Symlinks can't be dereferenced.
+            return os.path.dirname(executable)
 
     # Returns the path to the folder the app is installed _in_, e.g.: '/root/blah/programs'
     @staticmethod
