@@ -1,4 +1,4 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import numpy
@@ -6,25 +6,34 @@ import numpy.linalg
 import math
 from UM.Math.Float import Float
 
+from typing import Optional, Any, TYPE_CHECKING, cast
+
+if TYPE_CHECKING:
+    from UM.Math.Matrix import Matrix
+
 # Disable divide-by-zero warnings so that 1.0 / (1.0, 0.0, 0.0) returns (1.0, Inf, Inf) without complaining
 numpy.seterr(divide="ignore")
 
 
-##  Simple 3D-vector class based on numpy arrays.
-#
-#   This class represents an immutable 3-dimensional vector.
-class Vector(object):
+class Vector:
+    """Simple 3D-vector class based on numpy arrays.
+
+    This class represents an immutable 3-dimensional vector.
+    """
+
     # These fields are filled in below. This is needed to help static analysis tools (read: PyCharm)
     Null = None     # type: Vector
     Unit_X = None   # type: Vector
     Unit_Y = None   # type: Vector
     Unit_Z = None   # type: Vector
 
-    ##  Initialize a new vector
-    #   \param x X coordinate of vector.
-    #   \param y Y coordinate of vector.
-    #   \param z Z coordinate of vector.
-    def __init__(self, x = None, y = None, z = None, data = None, round_digits = None):
+    def __init__(self, x: Optional[float] = None, y: Optional[float] = None, z: Optional[float] = None, data: Optional[numpy.ndarray] = None, round_digits: Optional[int] = None) -> None:
+        """Initialize a new vector
+        :param x: X coordinate of vector.
+        :param y: Y coordinate of vector.
+        :param z: Z coordinate of vector.
+        """
+
         if x is not None and y is not None and z is not None:
             self._data = numpy.array([x, y, z], dtype = numpy.float64)
         elif data is not None:
@@ -33,44 +42,50 @@ class Vector(object):
             self._data = numpy.zeros(3, dtype = numpy.float64)
         self.round_digits = round_digits  # for comparisons
 
-    ##  Get numpy array with the data
-    #   \returns numpy array of length 3 holding xyz data.
-    def getData(self):
+    def getData(self) -> numpy.ndarray:
+        """Get numpy array with the data
+        :returns: numpy array of length 3 holding xyz data.
+        """
+
         return self._data.astype(numpy.float64)
 
-    def setRoundDigits(self, digits):
+    def setRoundDigits(self, digits: int) -> None:
         self.round_digits = digits
 
-    ##  Return the x component of this vector
     @property
     def x(self):
+        """Return the x component of this vector"""
+
         return numpy.float64(self._data[0])
 
-    ##  Return the y component of this vector
     @property
     def y(self):
+        """Return the y component of this vector"""
+
         return numpy.float64(self._data[1])
 
-    ## Return the z component of this vector
     @property
     def z(self):
+        """Return the z component of this vector"""
+
         return numpy.float64(self._data[2])
 
-    def set(self, x = None, y = None, z = None):
+    def set(self, x: Optional[float] = None, y: Optional[float] = None, z: Optional[float] = None) -> "Vector":
         new_x = self._data[0] if x is None else x
         new_y = self._data[1] if y is None else y
         new_z = self._data[2] if z is None else z
         return Vector(new_x, new_y, new_z)
 
-    ##  Get the angle from this vector to another
-    def angleToVector(self, vector):
+    def angleToVector(self, vector: "Vector") -> float:
+        """Get the angle from this vector to another"""
+
         v0 = numpy.array(self._data, dtype = numpy.float64, copy = False)
         v1 = numpy.array(vector.getData(), dtype = numpy.float64, copy = False)
         dot = numpy.sum(v0 * v1)
         dot /= self._normalizeVector(v0) * self._normalizeVector(v1)
         return numpy.arccos(numpy.fabs(dot))
-    
-    def normalized(self):
+
+    def normalized(self) -> "Vector":
         l = self.length()
         if l != 0:
             new_data = self._data / l
@@ -78,52 +93,55 @@ class Vector(object):
         else:
             return self
 
-    ##  Return length, i.e. Euclidean norm, of ndarray along axis.
-    def _normalizeVector(self, data):
+    def _normalizeVector(self, data: numpy.ndarray) -> numpy.ndarray:
+        """Return length, i.e. Euclidean norm, of ndarray along axis."""
+
         data = numpy.array(data, dtype = numpy.float64, copy = True)
         if data.ndim == 1:
-            return math.sqrt(numpy.dot(data, data))
+            return numpy.array([math.sqrt(numpy.dot(data, data))])
         data *= data
         out = numpy.atleast_1d(numpy.sum(data))
         numpy.sqrt(out, out)
         return out
 
-    def length(self):
+    def length(self) -> float:
         return numpy.linalg.norm(self._data)
 
-    def dot(self, other):
+    def dot(self, other) -> numpy.ndarray:
         return numpy.dot(self._data, other._data)
 
-    def cross(self, other):
+    def cross(self, other) -> "Vector":
         result = numpy.cross(self._data, other._data)
         return Vector(result[0], result[1], result[2])
 
-    def multiply(self, matrix):
+    def multiply(self, matrix: "Matrix") -> "Vector":
         d = numpy.empty(4, dtype = numpy.float64)
         d[0] = self._data[0]
         d[1] = self._data[1]
         d[2] = self._data[2]
         d[3] = 1.0
 
-        d = d.dot(matrix.getData())
+        d = cast(numpy.ndarray, d.dot(matrix.getData()))
 
         return Vector(d[0], d[1], d[2])
 
-    def preMultiply(self, matrix):
+    def preMultiply(self, matrix: "Matrix") -> "Vector":
         d = numpy.empty(4, dtype = numpy.float64)
         d[0] = self._data[0]
         d[1] = self._data[1]
         d[2] = self._data[2]
         d[3] = 1.0
 
-        d = matrix.getData().dot(d)
+        d = cast(numpy.ndarray, matrix.getData().dot(d))
 
         return Vector(d[0], d[1], d[2])
 
-    ##  Scale a vector by another vector.
-    #
-    #   This will do a component-wise multiply of the two vectors.
-    def scale(self, other):
+    def scale(self, other: "Vector") -> "Vector":
+        """Scale a vector by another vector.
+
+        This will do a component-wise multiply of the two vectors.
+        """
+
         return Vector(self.x * other.x, self.y * other.y, self.z * other.z)
 
     def __eq__(self, other):
@@ -133,11 +151,13 @@ class Vector(object):
             return False
         return self.equals(other)
 
-    ## Compares this vector to another vector.
-    #
-    #   \param epsilon optional tolerance value for the comparision.
-    #   \returns True if the two vectors are the same.
-    def equals(self, other, epsilon = 1e-6):
+    def equals(self, other: "Vector", epsilon: float = 1e-6) -> bool:
+        """Compares this vector to another vector.
+
+        :param epsilon: optional tolerance value for the comparision.
+        :returns: True if the two vectors are the same.
+        """
+
         return Float.fuzzyCompare(self.x, other.x, epsilon) and \
                Float.fuzzyCompare(self.y, other.y, epsilon) and \
                Float.fuzzyCompare(self.z, other.z, epsilon)
@@ -200,10 +220,10 @@ class Vector(object):
         return Vector(data = -1 * self._data)
 
     def __repr__(self):
-        return "Vector({0:.3}, {1:.3}, {2:.3})".format(self._data[0], self._data[1], self._data[2])
+        return "Vector({0:.3f}, {1:.3f}, {2:.3f})".format(self._data[0], self._data[1], self._data[2])
 
     def __str__(self):
-        return "<{0:.3},{1:.3},{2:.3}>".format(self._data[0], self._data[1], self._data[2])
+        return "<{0:.3f},{1:.3f},{2:.3f}>".format(self._data[0], self._data[1], self._data[2])
 
     def __lt__(self, other):
         return self._data[0] < other._data[0] and self._data[1] < other._data[1] and self._data[2] < other._data[2]
@@ -230,9 +250,10 @@ class Vector(object):
                 round(self._data[2], self.round_digits) >= round(other._data[2], self.round_digits))
 
 
-def isNumber(value):
+def isNumber(value: Any) -> bool:
     return type(value) in [float, int, numpy.float32, numpy.float64]
 
+
 Vector.Null = Vector()
 Vector.Unit_X = Vector(1, 0, 0)
 Vector.Unit_Y = Vector(0, 1, 0)
