@@ -1,5 +1,7 @@
-# Copyright (c) 2016 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
+
+from enum import IntEnum
 from typing import Optional, Union, Dict, List
 
 from PyQt5.QtCore import QTimer, pyqtSignal, QObject
@@ -12,7 +14,6 @@ from UM.Signal import Signal, signalemitter
 class Message(QObject):
     """Class for displaying messages to the user."""
 
-
     class ActionButtonStyle:
         DEFAULT = 0
         LINK = 1
@@ -22,9 +23,15 @@ class Message(QObject):
         ALIGN_LEFT = 2
         ALIGN_RIGHT = 3
 
+    class MessageType(IntEnum):
+        POSITIVE = 0
+        NEUTRAL = 1
+        WARNING = 2
+        ERROR = 3
+
     def __init__(self, text: str = "", lifetime: int = 30, dismissable: bool = True, progress: float = None,
                  title: Optional[str] = None, parent=None, use_inactivity_timer: bool = True, image_source: str = "",
-                 image_caption: str = "", option_text: str = "", option_state: bool = True) -> None:
+                 image_caption: str = "", option_text: str = "", option_state: bool = True, message_type: MessageType = MessageType.NEUTRAL) -> None:
 
         """Class for displaying messages to the user.
         Even though the lifetime can be set, in certain cases it can still have a lifetime if nothing happens with the
@@ -38,7 +45,7 @@ class Message(QObject):
         :param text: Text that needs to be displayed in the message
         :param lifetime: How long should the message be displayed (in seconds).
             if lifetime is 0, it will never automatically be destroyed.
-        :param dismissible: Can the user dismiss the message?
+        :param dismissable: Can the user dismiss the message?
         :param title: Phrase that will be shown above the message.
         :param image_source: an absolute path where an image can be found to be
         displayed (QUrl.toLocalFile()) can be used for that.
@@ -46,6 +53,9 @@ class Message(QObject):
         really, it's up to the QML to handle that).
         :param progress: Is there any progress to be displayed? if -1, it's seen
         as indeterminate.
+        :param message_type: Defines the type of message according to the MessageType enum (POSITIVE, NEUTRAL, WARNING,
+        ERROR, default: NEUTRAL). Depending on the type, an icon appears next to the message title. The NEUTRAL messages
+        contain no icon.
         """
 
         super().__init__(parent)
@@ -76,10 +86,18 @@ class Message(QObject):
         self._actions = []  # type: List[Dict[str, Union[str, int]]]
         self._title = title
 
+        self._message_type = message_type
+
+        self.actionTriggered.connect(self._onActionTriggered)
+
     # We use these signals as QTimers need to be triggered from a qThread. By using signals to pass it,
     # the events are forced to be on the event loop (which is a qThread)
     inactivityTimerStop = pyqtSignal()
     inactivityTimerStart = pyqtSignal()
+    # Our signals work a bit differently, but also don't support lambdas. As such, the pyqt version of the same signal
+    # is also added.
+    # The actionTriggered signal will trigger the PyQtAction triggered signal
+    pyQtActionTriggered = pyqtSignal(QObject, str)
     actionTriggered = Signal()
     optionToggled = Signal()
 
@@ -87,6 +105,9 @@ class Message(QObject):
     textChanged = Signal()
     progressChanged = Signal()
 
+    def _onActionTriggered(self, message, action):
+        self.pyQtActionTriggered.emit(message, action)
+
     def _stopInactivityTimer(self) -> None:
         if self._inactivity_timer:
             self._inactivity_timer.stop()
@@ -187,6 +208,14 @@ class Message(QObject):
     def getImageCaption(self) -> str:
         return self._image_caption
 
+    def getMessageType(self) -> MessageType:
+        """
+        Gets the type of the message.
+        The message gets a different icon according to its type.
+        :return: The type of the message (POSITIVE, NEUTRAL, WARNING, ERROR)
+        """
+        return self._message_type
+
     def setText(self, text: str) -> None:
         """Changes the text on the message.
 
