@@ -1,7 +1,8 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
+from typing import Union
 
-from PyQt5.QtCore import Qt
+from PyQt5.QtCore import Qt, QUrl
 
 from UM.Application import Application
 from UM.FlameProfiler import pyqtSlot
@@ -23,6 +24,7 @@ class VisibleMessagesModel(ListModel):
     ImageCaptionRole = Qt.UserRole + 12
     OptionTextRole = Qt.UserRole + 13
     OptionStateRole = Qt.UserRole + 14
+    MessageTypeRole = Qt.UserRole + 15
 
     def __init__(self, parent=None):
         super().__init__(parent)
@@ -39,6 +41,7 @@ class VisibleMessagesModel(ListModel):
         self.addRoleName(self.ImageCaptionRole, "image_caption")
         self.addRoleName(self.OptionTextRole, "option_text")
         self.addRoleName(self.OptionStateRole, "option_state")
+        self.addRoleName(self.MessageTypeRole, "message_type")
         self._populateMessageList()
 
     def _populateMessageList(self):
@@ -54,15 +57,24 @@ class VisibleMessagesModel(ListModel):
             "actions": self.createActionsModel(message.getActions()),
             "dismissable": message.isDismissable(),
             "title": message.getTitle(),
-            "image_source": message.getImageSource(),
+            "image_source": self.getImageSourceAsQUrl(message.getImageSource()),
             "image_caption": message.getImageCaption(),
             "option_text": message.getOptionText(),
-            "option_state": message.getOptionState()
+            "option_state": message.getOptionState(),
+            "message_type": int(message.getMessageType())
         })
         message.titleChanged.connect(self._onMessageTitleChanged)
         message.textChanged.connect(self._onMessageTextChanged)
         message.progressChanged.connect(self._onMessageProgress)
 
+    @staticmethod
+    def getImageSourceAsQUrl(image_source: Union[QUrl, str]) -> QUrl:
+        if type(image_source) is str:
+            return QUrl.fromLocalFile(image_source)
+        elif type(image_source) is QUrl:
+            return image_source
+        return QUrl.fromLocalFile("")
+
     def createActionsModel(self, actions):
         model = ListModel()
         model.addRoleName(self.IDRole, "action_id")
