@@ -8,6 +8,7 @@ from UM.FlameProfiler import pyqtSlot
 from datetime import timedelta
 import math
 
+from UM.Logger import Logger
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("uranium")
 
@@ -86,13 +87,18 @@ class Duration(QObject):
             self._minutes = -1
             self._seconds = -1
         else:
-            duration = round(duration)
+            try:
+                duration = round(duration)
+            except OverflowError:
+                Logger.log("w", "Duration was too large to convert, so resetting it.")
+                duration = 0
             # If a Python int goes above the upper bound of C++ int, which is 2^16 - 1, you will get a error when Qt
             # tries to convert the Python int to C++ int:
             #    TypeError: unable to convert a Python 'int' object to a C++ 'int' instance
             # So we make sure here that the number won't exceed the limit due to CuraEngine bug or whatever, and
             # Cura won't crash.
             if int(duration) >= (2**31):
+                Logger.log("w", "Duration was too large to convert, so resetting it.")
                 duration = 0
 
             self._days = math.floor(duration / (3600 * 24))
