@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import tempfile
@@ -6,20 +6,31 @@ import os
 import os.path
 import sys
 from typing import Union, IO
-
+fsync = os.fsync
 if sys.platform != "win32":
     import fcntl
-
     def lockFile(file):
-        fcntl.flock(file, fcntl.LOCK_EX)
-else:
+        try:
+            fcntl.flock(file, fcntl.LOCK_EX)
+        except OSError:  # Some file systems don't support file locks.
+            pass
+        except ValueError:  # Can fail on some operating systems due to other instances of the application closing the file to prepare for deletion.
+            pass  # It's okay to ignore the lock then because they were about to delete the file anyway.
+
+    if hasattr(fcntl, 'F_FULLFSYNC'):
+        def fsync(fd):
+            # https://lists.apple.com/archives/darwin-dev/2005/Feb/msg00072.html
+            # https://developer.apple.com/library/mac/documentation/Darwin/Reference/ManPages/man2/fsync.2.html
+            fcntl.fcntl(fd, fcntl.F_FULLFSYNC)
+
+else:  # On Windows, flock doesn't exist so we disable it at risk of corruption when using multiple application instances.
     def lockFile(file): #pylint: disable=unused-argument
         pass
 
 
 class SaveFile:
     """A class to handle atomic writes to a file.
-    
+
     This class can be used to perform atomic writes to a file. Atomic writes ensure
     that the file contents are always correct and that concurrent writes do not
     end up writing to the same file at the same time.
@@ -48,6 +59,8 @@ class SaveFile:
         return self._temp_file
 
     def __exit__(self, exc_type, exc_value, traceback):
+        self._temp_file.flush()
+        fsync(self._temp_file.fileno())
         self._temp_file.close()
 
         self.__max_retries = 10
