@@ -11,6 +11,7 @@ from UM.Math.Plane import Plane
 from UM.Math.Quaternion import Quaternion
 from UM.Math.Vector import Vector
 from UM.Message import Message
+from UM.Operations.GravityOperation import GravityOperation
 from UM.Operations.GroupedOperation import GroupedOperation
 from UM.Operations.LayFlatOperation import LayFlatOperation
 from UM.Operations.RotateOperation import RotateOperation
@@ -197,7 +198,7 @@ class RotateTool(Tool):
                 if id in self._handle.getExtraWidgetsColorMap() and self._active_widget == self._handle.ExtraWidgets(id):
                     axis = math.floor((self._active_widget.value - self._active_widget.XPositive90.value) / 2)
 
-                    angle = math.radians(-90 if self._active_widget.value - 2 * axis else 90)
+                    angle = math.radians(90 if (self._active_widget.value - ToolHandle.AllAxis) % 2 else -90)
                     axis +=  self._handle.XAxis
 
                     rotation = Quaternion()
@@ -266,10 +267,12 @@ class RotateTool(Tool):
             return
 
         rotate_operation = RotateOperation(current_node, rotation_quaternion, rotation_point_vector)
+        gravity_operation = GravityOperation(current_node)
         operation.addOperation(rotate_operation)
+        operation.addOperation(gravity_operation)
         operation.push()
 
-        # NOTE: We might want to consider unchecking the select-face button afterthe operation is done.
+        # NOTE: We might want to consider unchecking the select-face button after the operation is done.
 
     def getToolHint(self):
         """Return a formatted angle of the current rotate operation
@@ -349,7 +352,10 @@ class RotateTool(Tool):
         """
 
         self.operationStarted.emit(self)
-        self._progress_message = Message(i18n_catalog.i18nc("@label", "Laying object flat on buildplate..."), lifetime = 0, dismissable = False, title = i18n_catalog.i18nc("@title", "Object Rotation"))
+        self._progress_message = Message(i18n_catalog.i18nc("@label", "Laying object flat on buildplate..."),
+                                         lifetime = 0,
+                                         dismissable = False,
+                                         title = i18n_catalog.i18nc("@title", "Object Rotation"))
         self._progress_message.setProgress(0)
 
         self._iterations = 0
