@@ -1,3 +1,5 @@
+from unittest.mock import patch
+
 import pytest
 
 from UM.Scene.ToolHandle import ToolHandle
@@ -10,7 +12,8 @@ test_validate_data = [
 
 @pytest.mark.parametrize("data", test_validate_data)
 def test_getAndSet(data):
-    tool_handle = ToolHandle()
+    with patch("UM.Application.Application.getInstance"):
+        tool_handle = ToolHandle()
 
     # Attempt to set the value
     getattr(tool_handle, "set" + data["attribute"])(data["value"])
