@@ -38,6 +38,9 @@ def container_registry(application):
 
     ContainerRegistry.getInstance()._containers = {} # clear containers from previous iteration
 
+    root_plugin_dir = os.path.join(os.path.dirname(__file__), "..", "..", "..", "plugins")
+    PluginRegistry.getInstance().addPluginLocation(root_plugin_dir)
+
     PluginRegistry.getInstance().loadPlugin("LocalContainerProvider")
     plugin = PluginRegistry.getInstance().getPluginObject("LocalContainerProvider")
     ContainerRegistry.getInstance()._providers.append(plugin)
@@ -50,7 +53,7 @@ def container_registry(application):
     empty_container = ContainerRegistry.getInstance().getEmptyInstanceContainer()
     empty_definition_changes_container = empty_container
     empty_definition_changes_container.setMetaDataEntry("id", "empty_definition_changes")
-    empty_definition_changes_container.addMetaDataEntry("type", "definition_changes")
+    empty_definition_changes_container.setMetaDataEntry("type", "definition_changes")
     ContainerRegistry.getInstance().addContainer(empty_definition_changes_container)
 
     ContainerRegistry.getInstance().load()
